/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "resource.h"
#include "AccountSettings.h"
#include "MainSyncFrm.h"
#include "SyncForm.h"
#include "ProxySettings.h"

#include "winmaincpp.h"
#include "OutlookConfig.h"
#include "utils.h"
#include "comutil.h"
#include "OutlookPlugin.h"


IMPLEMENT_DYNCREATE(CAccountSettings, CFormView)

CAccountSettings::CAccountSettings()
	: CFormView(CAccountSettings::IDD)
{
}

CAccountSettings::~CAccountSettings()
{
}

void CAccountSettings::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_ACCOUNT_EDIT_URL, editUrl);
    DDX_Control(pDX, IDC_ACCOUNT_EDIT_USERNAME, editUser);
    DDX_Control(pDX, IDC_ACCOUNT_EDIT_PASSWORD, editPassword);
    DDX_Control(pDX, IDC_ACCOUNT_BUT_PROXY, butProxy);
    DDX_Control(pDX, IDC_ACCOUNT_GROUP_SERVER, groupServer);
}

BEGIN_MESSAGE_MAP(CAccountSettings, CFormView)
    ON_WM_CREATE()
    ON_BN_CLICKED(IDC_ACCOUNT_BUTOK, &CAccountSettings::OnBnClickedAccountButOk)
    ON_MESSAGE( WM_INITDIALOG, OnInitForm ) 
    ON_BN_CLICKED(IDC_ACCOUNT_BUT_CANCEL, &CAccountSettings::OnBnClickedAccountButCancel)
    //ON_WM_DESTROY()
    ON_WM_NCPAINT()
    ON_BN_CLICKED(IDC_ACCOUNT_BUT_PROXY, &CAccountSettings::OnBnClickedAccountButProxy)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


#ifdef _DEBUG
void CAccountSettings::AssertValid() const
{
	CFormView::AssertValid();
}

#ifndef _WIN32_WCE
void CAccountSettings::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif
#endif //_DEBUG


// CAccountSettings message handlers
int CAccountSettings::OnCreate(LPCREATESTRUCT lpcs){
    CFormView::OnCreate(lpcs);
    return 0;
}

LRESULT CAccountSettings::OnInitForm(WPARAM, LPARAM){
    CFormView::OnInitialUpdate(); //!! 
        
    const char* proxyUser = NULL;
    const char* proxyPass = NULL;
    CString s1, s2;

    // text limited to EDIT_TEXT_MAXLENGTH chars
    editUrl.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editUser.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editPassword.SetLimitText(EDIT_TEXT_MAXLENGTH);
    
    // load string resources
    s1.LoadString(IDS_ACCOUNT_URL);     SetDlgItemText(IDC_ACCOUNT_STATIC_URL, s1);
    s1.LoadString(IDS_ACCOUNT_USERNAME); SetDlgItemText(IDC_ACCOUNT_STATIC_USERNAME, s1);
    s1.LoadString(IDS_ACCOUNT_PASSWORD); SetDlgItemText(IDC_ACCOUNT_STATIC_PASSWORD, s1);
    s1.LoadString(IDS_SERVER); SetDlgItemText(IDC_ACCOUNT_GROUP_SERVER, s1);
    s1.LoadString(IDS_PROXY_SETTINGS); SetDlgItemText(IDC_ACCOUNT_BUT_PROXY, s1);
    
    s1.LoadString(IDS_OK); SetDlgItemText(IDC_ACCOUNT_BUTOK, s1);
    s1.LoadString(IDS_CANCEL); SetDlgItemText(IDC_ACCOUNT_BUT_CANCEL, s1);

    // read config from Registry 
    OutlookConfig* conf = (OutlookConfig*)getConfig();

    // Note: use 'toWideChar' because we need UTF-8 conversion.
    WCHAR* tmp = toWideChar(conf->getAccessConfig().getSyncURL());
    s2 = tmp; 
    delete [] tmp;
    SetDlgItemText(IDC_ACCOUNT_EDIT_URL, s2);

    // Note: use 'toWideChar' because we need UTF-8 conversion.
    tmp = toWideChar(conf->getAccessConfig().getUsername());
    s2 = tmp; 
    delete [] tmp;
    SetDlgItemText(IDC_ACCOUNT_EDIT_USERNAME, s2);

    // Note: use 'toWideChar' because we need UTF-8 conversion.
    tmp = toWideChar(conf->getAccessConfig().getPassword());
    s2 = tmp; 
    delete [] tmp;
    SetDlgItemText(IDC_ACCOUNT_EDIT_PASSWORD, s2);

    //editUrl.SetFocus();
    VERIFY(brushHollow.CreateStockObject(HOLLOW_BRUSH));

    groupServer.id = 2; 
    //groupServer.text.LoadString(IDS_SERVER);

    // disable windows xp theme, otherwise any color setting for groupbox
    // will be overriden by the theme settings
    if(((COutlookPluginApp*)AfxGetApp())->hLib){
        PFNSETWINDOWTHEME pfnSetWindowTheme =
            (PFNSETWINDOWTHEME)GetProcAddress(((COutlookPluginApp*)AfxGetApp())->hLib, "SetWindowTheme");
        pfnSetWindowTheme (groupServer.m_hWnd,L" ",L" ");
    }
    
    keepSettings = true;
    return 0;
};

void CAccountSettings::OnBnClickedAccountButCancel()
{
    keepSettings = false;
    getConfig()->read();  
    ((CMainSyncFrame*)AfxGetMainWnd())->pConfigFrame->DoCancel();
}


void CAccountSettings::OnBnClickedAccountButOk()
{
    keepSettings = false;
    if(saveSettings(true)){
        // -- close settings, show main window
        ((CMainSyncFrame*)AfxGetMainWnd())->pConfigFrame->DoCancel();
    }
}


bool CAccountSettings::saveSettings(bool saveToDisk)
{
    //-- check values
    CString url, username, password, proxyUser, proxyPassword;
    CString s1;
    _bstr_t bst;
    OutlookConfig* conf = (OutlookConfig*)getConfig();

    GetDlgItemText(IDC_ACCOUNT_EDIT_URL,url); 
    GetDlgItemText(IDC_ACCOUNT_EDIT_USERNAME,username);
    GetDlgItemText(IDC_ACCOUNT_EDIT_PASSWORD,password);

    if(url == ""){
        s1.LoadString(IDS_ERROR_SET_URL);
        MessageBox(s1);
        return false;
    };
    if(username == ""){
        s1.LoadString(IDS_ERROR_SET_USERNAME);
        MessageBox(s1);
        return false;
    };
    if(password == ""){
        s1.LoadString(IDS_ERROR_SET_PASSWORD);
        MessageBox(s1);
        return false;
    };


    // Note: use 'toMultibyte' which uses charset UTF-8.
    //       (when writing to winreg, toWideChar is then called)
    char* tmp = toMultibyte(url.GetBuffer());
    if (tmp) {
        conf->getAccessConfig().setSyncURL(tmp);
        delete [] tmp;
    }
    tmp = toMultibyte(username.GetBuffer());
    if (tmp) {
        conf->getAccessConfig().setUsername(tmp);
        delete [] tmp;
    }
    tmp = toMultibyte(password.GetBuffer());
    if (tmp) {
        conf->getAccessConfig().setPassword(tmp);
        delete [] tmp;
    }
    
    //-- save values to registry 
    if(saveToDisk)
        conf->save();

    return true;
}

void CAccountSettings::OnNcPaint(){
    CFormView::OnNcPaint();
    CScrollView::SetScrollSizes(MM_TEXT, CSize(0,0));   
}


void CAccountSettings::OnBnClickedAccountButProxy()
{
    CProxySettings wndProxySettings;
    INT_PTR result = wndProxySettings.DoModal();  
    //SetFocus();
}

BOOL CAccountSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_ESCAPE){
            OnBnClickedAccountButCancel();
            bProcessed = true;
        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CFormView::PreTranslateMessage(pMsg);
}


HBRUSH CAccountSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
    
    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomGroupbox)){
            pDC->SetBkMode(TRANSPARENT);
            pDC->SetTextColor(COLOR_GROUPBOX_BLACK);
        };
    
    return hbr;
}

void CAccountSettings::OnDestroy(){
    if(keepSettings)
        saveSettings(false);
}