/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "resource.h"
#include "TaskSettings.h"
#include "MainSyncFrm.h"
#include "ClientUtil.h"

#include "winmaincpp.h"
#include "utils.h"
#include "comutil.h"
#include "OutlookPlugin.h"

#include <string>

using namespace std;

// CTaskSettings

static wstring outlookFolder;
static CTaskSettings* wndTasks;
static HANDLE handleThread;

IMPLEMENT_DYNCREATE(CTaskSettings, CDialog)

CTaskSettings::CTaskSettings()
	: CDialog(CTaskSettings::IDD)
{

}

CTaskSettings::~CTaskSettings()
{
    // clean stuff used in the select Outlook folder thread
    wndTasks = NULL;
    if (handleThread) {
        CloseHandle(handleThread);
        handleThread = NULL;
    }
}

void CTaskSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_TASKS_COMBO_SYNCTYPE, lstSyncType);
    DDX_Control(pDX, IDC_TASKS_EDIT_FOLDER, editFolder);
    DDX_Control(pDX, IDC_TASKS_CHECK_INCLUDE, checkInclude);
    DDX_Control(pDX, IDC_TASKS_BUT_SELECT, butSelectFolder);
    DDX_Control(pDX, IDC_TASKS_EDIT_REMOTE, editRemote);
    DDX_Control(pDX, IDC_TASKS_GROUP_DIRECTION, groupDirection);
    DDX_Control(pDX, IDC_TASKS_GROUP_FOLDER, groupFolder);
    DDX_Control(pDX, IDC_TASKS_ADVANCED, groupAdvanced);
}

BEGIN_MESSAGE_MAP(CTaskSettings, CDialog)
    ON_BN_CLICKED(IDC_TASKS_OK, &CTaskSettings::OnBnClickedTasksOk)
    ON_BN_CLICKED(IDC_TASKS_CANCEL, &CTaskSettings::OnBnClickedTasksCancel)
    ON_BN_CLICKED(IDC_TASKS_BUT_SELECT, &CTaskSettings::OnBnClickedTasksButSelect)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CTaskSettings diagnostics

#ifdef _DEBUG
void CTaskSettings::AssertValid() const
{
	CDialog::AssertValid();
}

#ifndef _WIN32_WCE
void CTaskSettings::Dump(CDumpContext& dc) const
{
	CDialog::Dump(dc);
}
#endif
#endif //_DEBUG


// CTaskSettings message handlers
BOOL CTaskSettings::OnInitDialog(){
    CString s1;
    s1.LoadString(IDS_TASKS_DETAILS); SetWindowText(s1);
    CDialog::OnInitDialog();
    

    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(TASK_);

    editFolder.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editRemote.SetLimitText(EDIT_TEXT_MAXLENGTH);

    // load string resources
    s1.LoadString(IDS_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE2); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE3); lstSyncType.AddString(s1);

    s1.LoadString(IDS_SYNC_DIRECTION); SetDlgItemText(IDC_TASKS_GROUP_DIRECTION, s1);
    s1.LoadString(IDS_TASKS_FOLDER); SetDlgItemText(IDC_TASKS_GROUP_FOLDER, s1);
    s1.LoadString(IDS_CURRENT); SetDlgItemText(IDC_TASKS_STATIC_FOLDER, s1);
    s1.LoadString(IDS_INCLUDE_SUBFOLDERS); SetDlgItemText(IDC_TASKS_CHECK_INCLUDE, s1);
    s1.LoadString(IDS_SELECT_FOLDER); SetDlgItemText(IDC_TASKS_BUT_SELECT, s1);
    s1.LoadString(IDS_REMOTE_NAME); SetDlgItemText(IDC_TASKS_STATIC_REMOTE, s1);

    s1.LoadString(IDS_ADVANCED); SetDlgItemText(IDC_TASKS_ADVANCED, s1);
    s1.LoadString(IDS_DATA_FORMAT); SetDlgItemText(IDC_TASKS_STATIC_DATAFORMAT, s1);
    s1.LoadString(IDS_USE_SIF); SetDlgItemText(IDC_TASKS_STATIC_SIF, s1);

    s1.LoadString(IDS_OK); SetDlgItemText(IDC_TASKS_OK, s1);
    s1.LoadString(IDS_CANCEL); SetDlgItemText(IDC_TASKS_CANCEL, s1);

    lstSyncType.SetCurSel(getSyncTypeIndex(ssconf->getSync()));

    // Get folder path.
    // Note: use 'toWideChar' because we need UTF-8 conversion.
    WCHAR* olFolder = toWideChar(ssconf->getFolderPath());
    s1 = olFolder;
    delete [] olFolder;
    if(s1 == ""){
        s1 = getDefaultFolderPath(TASK).data();
    }
    SetDlgItemText(IDC_TASKS_EDIT_FOLDER, s1);

    if(ssconf->getUseSubfolders())
        checkInclude.SetCheck(BST_CHECKED);

    // Note: use 'toWideChar' because we need UTF-8 conversion.
    WCHAR* remName = toWideChar(ssconf->getURI());
    s1 = remName;
    delete [] remName;
    SetDlgItemText(IDC_TASKS_EDIT_REMOTE, s1);

    wndTasks = this;

    // disable windows xp theme, otherwise any color setting for groupbox
    // will be overriden by the theme settings
    if(((COutlookPluginApp*)AfxGetApp())->hLib){
        PFNSETWINDOWTHEME pfnSetWindowTheme =
            (PFNSETWINDOWTHEME)GetProcAddress(((COutlookPluginApp*)AfxGetApp())->hLib, "SetWindowTheme");
        pfnSetWindowTheme (groupDirection.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupFolder.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupAdvanced.m_hWnd,L" ",L" ");
    };

    // Accessing Outlook, could be no more in foreground
    SetForegroundWindow();

    return FALSE;
}

void CTaskSettings::OnBnClickedTasksOk()
{
    // OK Button
    if (saveSettings(false)){
        CDialog::OnOK();
    }
    
}

void CTaskSettings::OnBnClickedTasksCancel()
{
    // Never read from winreg, will save when 'OK' is clicked on SyncSettings.
    // getConfig()->read();
    CDialog::OnCancel();
}


bool CTaskSettings::saveSettings(bool saveToDisk)
{
    CString remoteName, outlookFolder, syncType;
    CString s1;
    _bstr_t bst;
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(TASK_);

    GetDlgItemText(IDC_TASKS_EDIT_REMOTE, remoteName);
    GetDlgItemText(IDC_TASKS_EDIT_FOLDER, outlookFolder);

    // change values
    if(remoteName == ""){
        s1.LoadString(IDS_ERROR_SET_REMOTE_NAME);
        MessageBox(s1);
        return false;
    };

    // sync source enabled
    ssconf->setSync(getSyncTypeName(lstSyncType.GetCurSel()));
    
    // Note: use 'toMultibyte' which uses charset UTF-8.
    //       (when writing to winreg, toWideChar is then called)
    char* olFolder = toMultibyte(outlookFolder.GetBuffer());
    if (olFolder) {
        ssconf->setFolderPath(olFolder);
        delete [] olFolder;
    }

    if(checkInclude.GetCheck() == BST_CHECKED)
        ssconf->setUseSubfolders(true);
    else
        ssconf->setUseSubfolders(false);

    char* remName = toMultibyte(remoteName.GetBuffer());
    if (remName) {
        ssconf->setURI(remName);
        delete [] remName;
    }

    // Never save to winreg, will save when 'OK' is clicked on SyncSettings.
    //if(saveToDisk)
    //    ((OutlookConfig*)getConfig())->save();

    return true;
}

int pickFolderTasks(){
    CString s1;
    outlookFolder = pickOutlookFolder(TASK);
    if (wndTasks) {      // dialog could have been closed...
        if (outlookFolder != L""){
            s1 = outlookFolder.data();
            wndTasks->SetDlgItemText(IDC_TASKS_EDIT_FOLDER, s1);
        }
        wndTasks->SetForegroundWindow();
    }
    return 0;
}

void CTaskSettings::OnBnClickedTasksButSelect()
{
    outlookFolder = L"";
    handleThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)pickFolderTasks, NULL, 0, NULL);
}

HBRUSH CTaskSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomGroupbox)){
        // do not check the id
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(COLOR_GROUPBOX_BLACK);
    };

    return hbr;
}
