/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include "WindowsSyncSourceConfig.h"
#include "base/util/utils.h"


WindowsSyncSourceConfig::WindowsSyncSourceConfig() {

    initialize();
    // Please note that 's' pointer MUST be set!!
}

WindowsSyncSourceConfig::WindowsSyncSourceConfig(SyncSourceConfig* sc) {

    initialize();

    // Link SyncSourceConfig pointer with common properties:
    if (!sc) {
        sprintf(lastErrorMsg, "WindowsSyncSourceConfig initialized with a NULL pointer!");
        throw lastErrorMsg;
    }
    s = sc;
}

WindowsSyncSourceConfig::~WindowsSyncSourceConfig() {
    if (folderPath) {
        delete [] folderPath;
        folderPath = NULL;
    }
}


// Return the pointer to external SyncSourceConfig object 
// used for common properties.
SyncSourceConfig* WindowsSyncSourceConfig::getCommonConfig() {
    return s;
}




//
// Internal properties
//
_declspec(dllexport) const char* WindowsSyncSourceConfig::getFolderPath() const {
    return folderPath;
}
void WindowsSyncSourceConfig::setFolderPath(const char* v) {
    safeDelete(&folderPath);
    folderPath = stringdup(v);
}

_declspec(dllexport) bool WindowsSyncSourceConfig::getUseSubfolders() const {
    return useSubfolders;
}
void WindowsSyncSourceConfig::setUseSubfolders(bool v) {
    useSubfolders = v;
}


_declspec(dllexport) long WindowsSyncSourceConfig::getEndTimestamp() const {
    return endTimestamp;
}
void WindowsSyncSourceConfig::setEndTimestamp(long v) {
    endTimestamp = v;
}


bool WindowsSyncSourceConfig::getIsSynced() const {
    return isSynced;
}
void WindowsSyncSourceConfig::setIsSynced(bool v) {
    isSynced = v;
}



/* 
 * Copy Constructor
 */
WindowsSyncSourceConfig::WindowsSyncSourceConfig(const WindowsSyncSourceConfig& wsc) {

    if (&wsc == this) {
        return;
    }

    initialize();

    // WARNING! pointer to the same object!
    s = wsc.s;

    setFolderPath    (wsc.getFolderPath    ());
    setUseSubfolders (wsc.getUseSubfolders ()); 
    setEndTimestamp  (wsc.getEndTimestamp  ());
    setIsSynced      (wsc.getIsSynced      ());
}


/* 
 * Operator =
 */
WindowsSyncSourceConfig& WindowsSyncSourceConfig::operator = (const WindowsSyncSourceConfig& wsc) {

    initialize();

    // WARNING! pointer to the same object!
    s = wsc.s;

    setFolderPath    (wsc.getFolderPath    ());
    setUseSubfolders (wsc.getUseSubfolders ()); 
    setEndTimestamp  (wsc.getEndTimestamp  ());
    setIsSynced      (wsc.getIsSynced      ());

    return *this;
}



/**
 * Assign the internal SyncSourceConfig* pointer.
 */
void WindowsSyncSourceConfig::setCommonConfig(SyncSourceConfig* sc) {
    s = sc;
}



// Initialize members
void WindowsSyncSourceConfig::initialize() {

    s               = NULL;
    folderPath      = NULL;
    useSubfolders   = false;
    endTimestamp    = 0;
    isSynced        = false;
}
