/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_CLIENT_EXC
#define INCL_CLIENT_EXC


#include "base/fscapi.h"
#include "base/errors.h"

#include <string>


/*
 ******************************************************************
 * Class ClientException: defines errors for Client operations.
 ******************************************************************
*/
class ClientException {

private:

    // pointer to ClientException instance
    static ClientException* pinstance;


    // The message to display
    char*  errorMsg;
    // The same message in WCHAR
    WCHAR* werrorMsg;

    // The error code
    DWORD errorCode;

    // is this error critic for the application?
    bool fatal;

    // do we need a messageBox displayed?
    bool useMessageBox;


protected:

    // Constructor
    ClientException();


public:

    // Method to create the sole instance of ClientException
    static ClientException* getInstance();
    
    // Destructor
    ~ClientException();


    // Set data of exception
    void setExceptionData(const char* msg,
                          DWORD code       = NULL, 
                          bool fatalError  = false, 
                          bool needDisplay = false);


    // Clear data of exception
    void clear();


    // Methods to get exception informations
    const char*         getErrorMsg();
    const WCHAR*        wgetErrorMsg();
    const std::wstring  wstrgetErrorMsg();

    DWORD  getErrorCode();
    bool   isFatal();
    bool   needMessageBox();

};


// ---------- Static functions -----------

//use these functions to throw Client exceptions.
void throwClientException(const char* msg, 
                           DWORD code       = NULL, 
                           bool fatalError  = NULL,
                           bool needDisplay = NULL);

// To throw a fatal outlook exception.
void throwClientFatalException(const char* msg, 
                                DWORD code = NULL);

// Actions to execute when a Client exception occurs.
void manageClientException(ClientException* e);


#endif
